searchData={"content_type":"text/markdown","items":[{"doc":"Defines a RabbitMQ client.","ref":"Lepus.html","title":"Lepus","type":"behaviour"},{"doc":"defmodule MyApp.RabbitMQ do\n      use Lepus, client: Lepus.BasicClient\n    end\n\nAdd `Lepus.BasicClient` to your supervision tree (see the `Lepus.BasicClient` documentation).\n\nUse it to publish a message to Rabbit MQ:\n\n    MyApp.RabbitMQ.publish_json(\"my-exchange\", \"my-routing-key\", %{a: 1, b: 2})","ref":"Lepus.html#module-example-usage","title":"Example usage - Lepus","type":"behaviour"},{"doc":"For testing purposes you can use another `:client`.\nIt could be [Mox](https://hexdocs.pm/mox/Mox.html) mock.\n\n\n    defmodule MyApp.RabbitMQ do\n      use Lepus, client: Application.get_env(:my_app, __MODULE__, []) |> Keyword.fetch!(:client)\n    end\n\nIn the `prod.exs` or `dev.exs`:\n\n    config :my_app, MyApp.RabbitMQ, client: Lepus.BasicClient\n\nIn the `test.exs`:\n\n    config :my_app, MyApp.RabbitMQ, client: MyApp.RabbitMQClientMock\n\nIn the tests:\n\n    test \"uses Mox for Lepus\" do\n      MyApp.RabbitMQClientMock\n      |> expect(:publish_json, fn _, \"my-exchange\", \"my-routing-key\", payload, _opts ->\n        assert %{my_key: \"My Value\"} = payload\n      end)\n\n      MyApp.RabbitMQ.publish_json(\"my-exchange\", \"my-routing-key\", %{my_key: \"My Value\"})\n    end","ref":"Lepus.html#module-testing","title":"Testing - Lepus","type":"behaviour"},{"doc":"Publishes message to RabbitMQ exchange.","ref":"Lepus.html#c:publish/4","title":"Lepus.publish/4","type":"callback"},{"doc":"* `amqp_opts` - `AMQP.Basic.publish/5` options.\n* `rpc` - Defines if the message uses\n  [RPC](https://www.rabbitmq.com/tutorials/tutorial-six-elixir.html) pattern.\n  The default value is `false`.\n* `timeout` - number of milliseconds or `:infinity`.\n  Valid for [RPC](https://www.rabbitmq.com/tutorials/tutorial-six-elixir.html).\n  The default value is `5000`.","ref":"Lepus.html#c:publish/4-options","title":"Options - Lepus.publish/4","type":"callback"},{"doc":"Without `rpc: true`:\n\n* `:ok` - in case of success\n* `{:error, any}` in case of error\n\nWith `rpc: true`:\n\n* `{:ok, any}` - in case of success\n* `{:error, :timeout}` - in case of timeout error\n* `{:error, any}` - in case of error","ref":"Lepus.html#c:publish/4-returns","title":"Returns - Lepus.publish/4","type":"callback"},{"doc":"The same as `c:publish/4` but `payload` can be any type that is convertable to JSON (map, list, etc.).\nSends JSON string to RabbitMQ exchange.","ref":"Lepus.html#c:publish_json/4","title":"Lepus.publish_json/4","type":"callback"},{"doc":"Basic `Lepus.Client` implementation.","ref":"Lepus.BasicClient.html","title":"Lepus.BasicClient","type":"module"},{"doc":"* Supervises `AMQP.Connection` process and many `AMQP.Channel` processes\n  (one `AMQP.Channel` process per exchange).\n* Supports [RPC](https://www.rabbitmq.com/tutorials/tutorial-six-elixir.html)","ref":"Lepus.BasicClient.html#module-features","title":"Features - Lepus.BasicClient","type":"module"},{"doc":"Add it to your supervision tree\n\n    children = [\n      {\n        Lepus.BasicClient,\n        name: MyApp.RabbitMQ,\n        exchanges: [\"exchange1\", \"exchange2\"],\n        connection: [\n          host: \"localhost\",\n          port: 5672,\n          virtual_host: \"/\",\n          username: \"guest\",\n          password: \"guest\"\n        ],\n        rpc_opts: [\n          pubsub: MyApp.PubSub,\n          reply_to_queue: \"my_app.reply_to\"\n        ]\n      }\n    ]\n\n    children |> Supervisor.start_link(strategy: :one_for_one)","ref":"Lepus.BasicClient.html#module-example-usage","title":"Example usage - Lepus.BasicClient","type":"module"},{"doc":"* `:name` (`t:atom/0`) - Required. Module name of your client. If you use\n\n  ```elixir\n  defmodule MyApp.RabbitMQ do\n    use Lepus, client: Lepus.BasicClient\n  end\n  ```\n\n  the `name` should be `MyApp.RabbitMQ`.\n\n* `:connection` - Required. Defines an AMQP URI or a set of options used by the RabbitMQ client to open the connection\n  with the RabbitMQ broker.\n  See `AMQP.Connection.open/1` for the full list of options.\n\n* `:exchanges` (list of `t:String.t/0`) - List of exchange names you are going to use.\n  `Lepus.BasicClient` starts and supervises separate process per exchange.\n  Process for the default exchange (`\"\"`) is always started. The default value is `[]`.\n\n* `:rpc_opts` (`t:keyword/0`) - Required only if you are going to use\n  [RPC](https://www.rabbitmq.com/tutorials/tutorial-six-elixir.html)\n  (`c:Lepus.publish/4` or `c:Lepus.publish_json/4` with `rpc: true`). The default value is `[]`.\n\n  * `:pubsub` (`t:atom/0`) - Your PubSub process name. See the `Phoenix.PubSub`.\n\n  * `:reply_to_queue` (`t:String.t/0`) - Defines a queue name for the replies.\n\n  * `:queues_type` - Defines reply_to queue type (one of `[\"classic\", \"quorum\"]`). Used during queue declaration.","ref":"Lepus.BasicClient.html#module-options","title":"Options - Lepus.BasicClient","type":"module"},{"doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Lepus.BasicClient.html#child_spec/1","title":"Lepus.BasicClient.child_spec/1","type":"function"},{"doc":"","ref":"Lepus.BasicClient.html#start_link/1","title":"Lepus.BasicClient.start_link/1","type":"function"},{"doc":"Client behaviour used in `Lepus`.","ref":"Lepus.Client.html","title":"Lepus.Client","type":"behaviour"},{"doc":"","ref":"Lepus.Client.html#c:publish/5","title":"Lepus.Client.publish/5","type":"callback"},{"doc":"","ref":"Lepus.Client.html#c:publish_json/5","title":"Lepus.Client.publish_json/5","type":"callback"},{"doc":"","ref":"Lepus.Client.html#t:binary_payload/0","title":"Lepus.Client.binary_payload/0","type":"type"},{"doc":"","ref":"Lepus.Client.html#t:exchange/0","title":"Lepus.Client.exchange/0","type":"type"},{"doc":"","ref":"Lepus.Client.html#t:opts/0","title":"Lepus.Client.opts/0","type":"type"},{"doc":"","ref":"Lepus.Client.html#t:payload/0","title":"Lepus.Client.payload/0","type":"type"},{"doc":"","ref":"Lepus.Client.html#t:response/0","title":"Lepus.Client.response/0","type":"type"},{"doc":"","ref":"Lepus.Client.html#t:routing_key/0","title":"Lepus.Client.routing_key/0","type":"type"},{"doc":"Consumer receives messages from a RabbitMQ queue.\nIt uses `Broadway` and `BroadwayRabbitMQ.Producer` under the hood.","ref":"Lepus.Consumer.html","title":"Lepus.Consumer","type":"behaviour"},{"doc":"* Optionally retries failed messages with exponential backoff\n* Optionally publishes failed messages to separate queue\n* Supports [RPC](https://www.rabbitmq.com/tutorials/tutorial-six-elixir.html)","ref":"Lepus.Consumer.html#module-features","title":"Features - Lepus.Consumer","type":"behaviour"},{"doc":"```txt\n┌►   [queue]\n│       │\n│       ▼\n│ (Lepus.Consumer) ───► {failed exchange} ───► [failed queue]\n│       │\n│       ▼\n│ {delay exchange}\n│       │\n│       ▼\n│ [retry queue]\n│       │\n│       ▼\n└ {retry exchange}\n```","ref":"Lepus.Consumer.html#module-topology","title":"Topology - Lepus.Consumer","type":"behaviour"},{"doc":"Define a consumer:\n\n    defmodule MyApp.MyConsumer do\n      @behaviour Lepus.Consumer\n\n      @impl Lepus.Consumer\n      def options do\n        [\n          connection: [\n            host: \"localhost\",\n            port: 5672,\n            virtual_host: \"/\",\n            username: \"guest\",\n            password: \"guest\"\n          ],\n          exchange: \"my_exchange\",\n          routing_key: \"my_routing_key\",\n          queue: \"my_queue\",\n          store_failed: true,\n          max_retry_count: 5\n        ]\n      end\n\n      @impl Lepus.Consumer\n      def handle_message(data, metadata) do\n        Do.something_with(data, metadata)\n      end\n    end\n\nThen add it to your supervision tree (usually in `lib/my_app/application.ex`):\n\n    children = [\n      {Lepus.Consumer, consumers: [MyApp.MyConsumer]}\n    ]\n\n    children |> Supervisor.start_link(strategy: :one_for_one)\n\nYou can also define some global options for all the consumers in the supervision tree:\n\n    children = [\n      {Lepus.Consumer,\n        consumers: [MyApp.MyConsumer, MyApp.OtherConsumer],\n        options: [connection: rabbit_connection, exchange: \"my_exchange\"]}\n    ]\n\n    children |> Supervisor.start_link(strategy: :one_for_one)","ref":"Lepus.Consumer.html#module-example","title":"Example - Lepus.Consumer","type":"behaviour"},{"doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Lepus.Consumer.html#child_spec/1","title":"Lepus.Consumer.child_spec/1","type":"function"},{"doc":"Receives message `payload` and `metadata`.\n\n## `payload`\n\nUsually it's a binary RabbitMQ message payload.\nBut it could be parsed JSON (map, list, etc.)\nif content type of the message is \"application/json\" or it was sent via `c:Lepus.publish_json/4`\n\n## `metadata`\n\nIt's a map with the keys:\n\n* `rpc` - Defines if the message was sent using\n  [RPC](https://www.rabbitmq.com/tutorials/tutorial-six-elixir.html) pattern\n  (`reply_to` and `correlation_id` is not empty).\n  The function should return `{:ok, result}` if `rpc` is `true`.\n\n* `retry_count` - Count of retries if the message was retried. Otherwise `0`.\n\n* `retriable` – Defines if the message will be retried in case of error.\n\n* `client` – `\"lepus\"` if the message was sent via `c:Lepus.publish/4` or `c:Lepus.publish_json/4`.\n\n* `rabbit_mq_metadata` - `metadata` field from `BroadwayRabbitMQ.Producer`.","ref":"Lepus.Consumer.html#c:handle_message/2","title":"Lepus.Consumer.handle_message/2","type":"callback"},{"doc":"* `{:ok, response}` – so `c:Lepus.publish/4` or `c:Lepus.publish_json/4` returns `{:ok, response}`\n\n* `{:error, response}` – so `c:Lepus.publish/4` or `c:Lepus.publish_json/4` returns `{:error, response}`","ref":"Lepus.Consumer.html#c:handle_message/2-should-return-in-case-of-using-rpc","title":"Should return in case of using RPC: - Lepus.Consumer.handle_message/2","type":"callback"},{"doc":"* `:ok` – exists successfully.\n\n* `:error` – exits with error. The message could be retried if `max_retry_count > 0`.\n\n* `{:error, error_message}` – The same as `:error` but adds the `error_message` to RabbitMQ headers.\n\n* `:retry` – always retries message (even if `max_retry_count == 0`).\n\n* `{:retry, reason}` – The same as `:retry` but adds the `reason` to RabbitMQ headers.","ref":"Lepus.Consumer.html#c:handle_message/2-should-return-in-case-of-not-using-rpc","title":"Should return in case of not using RPC: - Lepus.Consumer.handle_message/2","type":"callback"},{"doc":"Should return keyword list of options (if defined).\nYou can also define options in the supervision tree as described above.","ref":"Lepus.Consumer.html#c:options/0","title":"Lepus.Consumer.options/0","type":"callback"},{"doc":"* `:connection` - Required. Defines an AMQP URI or a set of options used by the RabbitMQ client to open the connection\n  with the RabbitMQ broker.\n  See `AMQP.Connection.open/1` for the full list of options.\n\n* `:exchange` (`t:String.t/0`) - Required. The name of the RabbitMQ exchange.\n\n* `:routing_key` (`t:String.t/0`) - Required. The name of the RabbitMQ routing_key.\n\n* `:delay_exchange` (`t:String.t/0`) - Redefines delay exchange name. The default value is `\"#{exchange}.delay\"`\n\n* `:retry_exchange` (`t:String.t/0`) - Redefines retry exchange name. The default value is `\"#{exchange}.retry\"`\n\n* `:failed_exchange` (`t:String.t/0`) - Redefines error exchange name. The default value is `\"#{exchange}.failed\"`\n\n* `:queue` (`t:String.t/0`) - Redefines queue name. The default value is `\"#{exchange}.#{routing_key}\"`\n\n* `:retry_queue` (`t:String.t/0`) - Redefines retry queue name. The default value is `\"#{queue}.retry\"`\n\n* `:failed_queue` (`t:String.t/0`) - Redefines error queue name. The default value is `\"#{queue}.failed\"`\n\n* `:processor` (`t:keyword/0`) - Broadway processor options.\n  Only one processor is used.\n  See `Broadway.start_link/2` for the full list of options. The default value is `[]`.\n\n* `:max_retry_count` (`t:non_neg_integer/0`) - The maximum count of message retries. The default value is `0`.\n\n* `:store_failed` (`t:boolean/0`) - Defines if the failed message should be published to `failed_queue`. The default value is `false`.\n\n* `:queues_type` - Defines queues type (one of `[\"classic\", \"quorum\"]`). Used during queue declaration. The default value is `\"classic\"`.","ref":"Lepus.Consumer.html#c:options/0-options","title":"Options - Lepus.Consumer.options/0","type":"callback"},{"doc":"","ref":"Lepus.Consumer.html#start_link/1","title":"Lepus.Consumer.start_link/1","type":"function"},{"doc":"","ref":"Lepus.Consumer.html#t:metadata/0","title":"Lepus.Consumer.metadata/0","type":"type"},{"doc":"","ref":"Lepus.Consumer.html#t:payload/0","title":"Lepus.Consumer.payload/0","type":"type"},{"doc":"# Lepus\n\nOpinionated library for [RabbitMQ](https://www.rabbitmq.com/).","ref":"readme.html","title":"Lepus","type":"extras"},{"doc":"[HexDocs](https://hexdocs.pm/lepus)","ref":"readme.html#documentation","title":"Documentation - Lepus","type":"extras"},{"doc":"* Supervises `AMQP.Connection` and `AMQP.Channel` processes.\n* Optionally retries failed messages with exponential backoff.\n* Optionally publishes failed messages to separate queue.\n* Supports [RPC](https://www.rabbitmq.com/tutorials/tutorial-six-elixir.html).","ref":"readme.html#features","title":"Features - Lepus","type":"extras"},{"doc":"Add `:lepus` to the list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:lepus, \"~> 0.1.0\"},\n  ]\nend\n```","ref":"readme.html#installation","title":"Installation - Lepus","type":"extras"}]}